from abc import ABC, abstractmethod

class BaseWorker(ABC):
    def __init__():
        pass

    @abstractmethod
    def start(self):
        pass

    @abstractmethod
    def stop(self):
        pass
